package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysRole;
import com.qmth.teachcloud.common.entity.SysRolePrivilege;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.entity.SysUserRole;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.mapper.SysRoleMapper;
import com.qmth.teachcloud.common.service.CacheService;
import com.qmth.teachcloud.common.service.SysRolePrivilegeService;
import com.qmth.teachcloud.common.service.SysRoleService;
import com.qmth.teachcloud.common.service.SysUserRoleService;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.misc.Request;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private SysRolePrivilegeService sysRolePrivilegeService;

    @Autowired
    private CacheService cacheService;

    @Resource
    SysRoleService sysRoleService;

    @Override
    public IPage<SysRole> list(String name, Boolean enable, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.listPage(new Page<>(pageNumber, pageSize), schoolId, SystemConstant.translateSpecificSign(name), enable);
    }

    @Transactional
    @Override
    @Deprecated
    public boolean saveRole(SysRole role) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        role.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRole::getSchoolId, role.getSchoolId()).eq(SysRole::getName, role.getName());
        SysRole sysRole = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(role.getId())) {
            if (sysRole != null) {
                throw ExceptionResultEnum.ERROR.exception("角色已存在");
            }
            role.setId(SystemConstant.getDbUuid());
            role.setCreateId(sysUser.getId());
            role.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(role);
        }
        // 修改
        else {
            if (sysRole != null && sysRole.getId().longValue() != role.getId().longValue()) {
                throw ExceptionResultEnum.ERROR.exception("角色已存在");
            }

            role.setUpdateId(sysUser.getId());
            role.setUpdateTime(System.currentTimeMillis());

//            if (RoleTypeEnum.CUSTOM.equals(role.getType())) {
            //删除权限
            sysRolePrivilegeService.removeByRoleId(role.getId());
//            }
            isSuccess = this.updateById(role);

        }

//        if (RoleTypeEnum.CUSTOM.equals(role.getType())) {
        // 新增权限
        sysRolePrivilegeService.saveBatch(role);
//        }

        // 更新sys_user_role
        sysUserRoleService.rebindPrivileges(role.getId());

        // 清除缓存
        cacheService.removeUserAuthCache(sysUser.getId());
        cacheService.removeRolePrivilegeCache(role.getId());

        return isSuccess;
    }

    /**
     * 新增角色
     *
     * @param role
     * @return
     */
    @Transactional
    @Override
    public boolean saveRoleNew(SysRole role) {
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            if (Objects.nonNull(role.getId())) {//编辑
                List<SysRolePrivilege> sysRolePrivilegeList = cacheService.rolePrivilegeCache(role.getId());
                int count = (int) sysRolePrivilegeList.stream().filter(s -> Arrays.asList(role.getPrivilegeIds()).contains(s.getPrivilegeId())).count();
                role.setUpdateId(sysUser.getId());
                sysRoleService.updateById(role);
                cacheService.updateRoleCache(role.getId());
                if (count != sysRolePrivilegeList.size() || count != role.getPrivilegeIds().length) {
                    sysRolePrivilegeService.removeByRoleId(role.getId());
                    //角色权限
                    sysRolePrivilegeService.saveBatch(role);
                    cacheService.updateRolePrivilegeCache(role.getId());
                }
            } else {
                role.setCreateId(sysUser.getId());
                sysRoleService.save(role);
                sysRolePrivilegeService.saveBatch(role);
            }
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return true;
    }

    @Override
    public boolean enable(SysRole role) {
        UpdateWrapper<SysRole> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysRole::getEnable, role.getEnable()).eq(SysRole::getId, role.getId());
        this.update(updateWrapper);
        cacheService.roleCache(role.getId());
        return true;
    }

    @Transactional
    @Override
    public boolean remove(Long id) {
        // 判断绑定用户
        List<SysUserRole> userRoleList = sysUserRoleService.listByRoleId(id);
        if (userRoleList != null && userRoleList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("用户绑定，不能删除");
        }
        //删除权限
        sysRolePrivilegeService.removeByRoleId(id);
        this.removeById(id);
        cacheService.removeRoleCache(id);
        cacheService.removeRolePrivilegeCache(id);
        return true;
    }

    @Override
    public List<Long> getUserRoles(Long userId) {
        QueryWrapper<SysUserRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserRole::getUserId, userId);
        List<SysUserRole> sysUserRoles = sysUserRoleService.list(queryWrapper);
        List<Long> list = null;
        if (sysUserRoles != null && sysUserRoles.size() > 0) {
            list = sysUserRoles.stream().map(m -> m.getRoleId()).collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public List<SysRole> listRolesByUserId(Long id) {
        return this.baseMapper.listRolesByUserId(id);
    }

    @Override
    public List<SysRole> list(Long[] roleIds, String name) {
        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRole::getType, name).in(SysRole::getId, roleIds);
        return this.baseMapper.selectList(queryWrapper);
    }

    @Override
    public List<SysRole> listToUser() {
        Long schoolId = null;
        try {
            Object object = ServletUtil.getRequestHeaderSchoolId();
            schoolId = Long.valueOf(object.toString());
        } catch (Exception e) {

        }
        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        Long finalSchoolId = schoolId;
        queryWrapper.lambda().eq(SysRole::getEnable, true).and(e -> e.eq(SysRole::getSchoolId, finalSchoolId).or().isNull(SysRole::getSchoolId));
        return this.list(queryWrapper);
    }
}
